@@ -31,7 +31,7 @@ logger = logging.getLogger('hardshare')
 logger.setLevel(logging.WARNING)
 loghandler = logging.handlers.WatchedFileHandler(filename='hardshare_client.log', mode='a')
 loghandler.setLevel(logging.DEBUG)
-loghandler.setFormatter(logging.Formatter('%(name)s (%(levelname)s) (pid: {});'
+loghandler.setFormatter(logging.Formatter('%(name)s.%(funcName)s (%(levelname)s) (pid: {});'
                                           ' %(asctime)s ; %(message)s'
                                           .format(os.getpid())))
 logger.addHandler(loghandler)
@@ -195,8 +195,11 @@ def main(argv=None):
         except:
             print('error loading configuration data. does it exist?')
             return 1
-        findings = WorkspaceInstance.inspect_instance()
-        print(json.dumps(findings))
+        if len(config['wdeployments']) == 0:
+            findings = WorkspaceInstance.inspect_instance()
+        else:
+            findings = WorkspaceInstance.inspect_instance(wdeployment=config['wdeployments'][0])
+        print(json.dumps(findings, indent=4))
 
     elif argv_parsed.command == 'ad':
         if ac is None:
@@ -209,14 +212,26 @@ def main(argv=None):
             os.close(0)
             os.close(1)
             os.close(2)
-        ac.run_sync()
+        return ac.run_sync()
 
     elif argv_parsed.command == 'terminate':
         if argv_parsed.purge_supposed_instance:
-            findings = WorkspaceInstance.inspect_instance()
+            try:
+                config = get_local_config()
+            except:
+                print('error loading configuration data. does it exist?')
+                return 1
+            if len(config['wdeployments']) == 0:
+                print('ERROR: no workspace deployment defined! Try: hardshare check')
+                return 1
+            cprovider = config['wdeployments'][0]['cprovider']
+            if cprovider not in ['docker', 'podman']:
+                print('unknown cprovider: {}'.format(cprovider))
+                return 1
+            findings = WorkspaceInstance.inspect_instance(wdeployment=config['wdeployments'][0])
             if 'container' in findings:
                 try:
-                    subprocess.check_call(['docker', 'rm', '-f',
+                    subprocess.check_call([cprovider, 'rm', '-f',
                                            findings['container']['name']],
                                           stdout=subprocess.DEVNULL,
                                           stderr=subprocess.DEVNULL)
@@ -302,12 +317,13 @@ def main(argv=None):
                 print('\t(none)')
             else:
                 for wdeployment in config['wdeployments']:
-                    print('\t{} (owner: {})'.format(
+                    print('{}\n\towner: {}\n\tcprovider: {}'.format(
                         wdeployment['id'],
-                        wdeployment['owner'])
-                    )
+                        wdeployment['owner'],
+                        wdeployment['cprovider']
+                    ))
 
-            print('found keys:')
+            print('\nfound keys:')
             if len(config['keys']) == 0:
                 print('\t(none)')
             else:
@@ -334,9 +350,9 @@ def main(argv=None):
                         print(remote_config['err'])
                     return 1
                 if len(remote_config['deployments']) == 0:
-                    print('no registered workspace deployments with this user account')
+                    print('\nno registered workspace deployments with this user account')
                 else:
-                    print('registered workspace deployments with this user account:')
+                    print('\nregistered workspace deployments with this user account:')
                     for wd in remote_config['deployments']:
                         print('{}'.format(wd['id']))
                         print('\tcreated: {}'.format(wd['date_created']))
